package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.nio.file.Path;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.nio.file.OpenOption;
import java.nio.file.Paths;
import java.io.Closeable;
import org.apache.zookeeper.common.IOUtils;
import java.io.FileWriter;
import com.jcraft.jsch.SftpException;
import com.jcraft.jsch.JSchException;
import java.text.ParseException;
import java.io.IOException;
import de.hybris.platform.util.Config;
import java.util.Objects;
import java.io.File;
import java.util.TimeZone;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import de.hybris.platform.core.servicelayer.data.SearchPageData;
import java.util.Date;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.servicelayer.exceptions.ModelSavingException;
import com.bpcl.lpg.core.model.LpgSafetyInspectionModel;
import de.hybris.platform.tx.Transaction;
import java.util.stream.Collectors;
import java.util.Arrays;
import java.util.Map;
import java.util.Collection;
import org.apache.commons.collections.CollectionUtils;
import java.util.ArrayList;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import com.bpcl.lpg.core.service.LpgDeliveryConfirmationService;
import com.bpcl.lpg.core.encryption.pgp.DefaultLpgPgpKeys;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.model.ModelService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.lpg.core.dao.BpclSafetyInspectionDao;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class LpgSafetyInspectionExportJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private static final String namePrefix = "BPCL_SafetyInspection_";
    private static final String IST = "IST";
    private static final String SEPARETOR = "~";
    public static final String TEMP_DIR;
    private static final String TEMP = "temp_";
    private static final String SAFETYINSPECTION = "3";
    private static final String SURAKSHACHANGE = "2";
    private static final String ONE = "1";
    private BpclSafetyInspectionDao bpclSafetyInspectionDao;
    private ConfigurationService configurationService;
    private NotificationEngineService notificationEngineService;
    @Resource(name = "modelService")
    private ModelService modelService;
    @Resource(name = "defaultLpgPgpKeys")
    private DefaultLpgPgpKeys defaultLpgPgpKeys;
    private LpgDeliveryConfirmationService lpgDeliveryConfirmationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronjob) {
        final Date startDate = this.populateStartDate(cronjob);
        LpgSafetyInspectionExportJob.LOG.info("Date ranges starts from : {}", startDate);
        final Date endDate = this.populateEndDate(cronjob);
        LpgSafetyInspectionExportJob.LOG.info("Filter End Date : {}", endDate);
        int currentPage = 0;
        try {
            final int pageSize = this.configurationService.getConfiguration().getInt("bpcl.lpg.safety.inspection.outbound.page.size", 25000);
            SearchPageData<LpgSafetyInspectionModel> searchPageData = (SearchPageData<LpgSafetyInspectionModel>)this.bpclSafetyInspectionDao.getSafetyInspectionDetailsForExport(startDate, endDate, pageSize, currentPage);
            final int totalPages = searchPageData.getPagination().getNumberOfPages();
            LpgSafetyInspectionExportJob.LOG.info("Page size: {}, total pages: {}, total records: {}", pageSize, totalPages, searchPageData.getPagination().getTotalNumberOfResults());
            final List<Long> dupCheckList = new ArrayList<Long>();
            while (currentPage < totalPages) {
                searchPageData = (SearchPageData<LpgSafetyInspectionModel>)this.bpclSafetyInspectionDao.getSafetyInspectionDetailsForExport(startDate, endDate, pageSize, 0);
                final List<LpgSafetyInspectionModel> safetyInspectionList = searchPageData.getResults();
                LpgSafetyInspectionExportJob.LOG.info("safetyInspectionList {} ", (Object)(CollectionUtils.isNotEmpty((Collection)safetyInspectionList) ? safetyInspectionList.size() : 0));
                if (CollectionUtils.isNotEmpty((Collection)safetyInspectionList)) {
                    final List<LpgSafetyInspectionModel> safetyInspUniqueList = safetyInspectionList.stream().collect(Collectors.toMap(si -> Arrays.asList(si.getCustomerNumber(), si.getDistributorCode()), si -> si, (existing, replacement) -> existing)).values().stream().toList();
                    this.generateReportCSV(safetyInspUniqueList, dupCheckList);
                    final Transaction txn = Transaction.current();
                    txn.begin();
                    boolean success = false;
                    try {
                        for (final LpgSafetyInspectionModel safetyInspection : safetyInspectionList) {
                            LpgSafetyInspectionExportJob.LOG.info("Safety Inspection Data for customer {} Sent to LPGNEXT ", safetyInspection.getCustomerMobileNumber());
                            safetyInspection.setSentToLpgNxt(Boolean.valueOf(true));
                        }
                        this.modelService.saveAll((Collection)safetyInspectionList);
                        success = true;
                    }
                    catch (final ModelSavingException e) {
                        LpgSafetyInspectionExportJob.LOG.error("Exception Occured While updating safety Inspection sentToLpgNxt flag :" + e.getMessage());
                        throw new ModelSavingException("Exception Occured While updating LpgSafetyInspection");
                    }
                    finally {
                        if (success) {
                            txn.commit();
                        }
                        else {
                            txn.rollback();
                        }
                    }
                }
                ++currentPage;
                Thread.sleep(10000L);
            }
        }
        catch (final Exception ex) {
            LpgSafetyInspectionExportJob.LOG.error("CSV file Processing Exception : ", ex);
            final String body = MessageFormat.format("Safety Inspection export job failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Safety Inspection export Job Alert", body, this.configurationService.getConfiguration().getString("lpg.safety.inspection.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
        return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
    }
    
    private Date populateStartDate(final BpclCronjobModel cronjob) {
        Date startDate = cronjob.getFilterStartDate();
        if (startDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(10, -this.configurationService.getConfiguration().getInt("lpg.safety.inspection.export.data.start.period"));
            startDate = cal.getTime();
        }
        return startDate;
    }
    
    private Date populateEndDate(final BpclCronjobModel cronjob) {
        Date endDate = cronjob.getFilterEndDate();
        if (endDate == null) {
            final Calendar cal = Calendar.getInstance();
            endDate = cal.getTime();
        }
        return endDate;
    }
    
    private void generateReportCSV(final List<LpgSafetyInspectionModel> safetyInspectionList, final List<Long> dupCheckList) throws IOException, ParseException, JSchException, SftpException {
        final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("IST"));
        final String date = simpleDateFormat.format(new Date());
        final String fileExtension = ".csv";
        final String fileName = "BPCL_SafetyInspection_" + date + fileExtension;
        final String tempFileName = LpgSafetyInspectionExportJob.TEMP_DIR + "temp_" + fileName;
        final String path = this.configurationService.getConfiguration().getString("bpcl.safety.inspection.hybris.input.filepath") + fileName;
        File safetyInspectionFile = new File(path);
        final StringBuilder fileContant = new StringBuilder();
        this.writeCsvHeader(fileContant);
        fileContant.append(System.getProperty("line.separator"));
        for (final LpgSafetyInspectionModel safetyInspection : safetyInspectionList) {
            if (!dupCheckList.contains(safetyInspection.getId())) {
                this.writeCsvLine(fileContant, safetyInspection);
                if ("1".equalsIgnoreCase(String.valueOf(safetyInspection.getQuestionRes().charAt(4)))) {
                    this.writeCsvLineSuraksha(fileContant, safetyInspection);
                }
                dupCheckList.add(safetyInspection.getId());
            }
        }
        LpgSafetyInspectionExportJob.LOG.info("CSV File created Successfully and file name is {}", fileName);
        if (Objects.nonNull(fileContant)) {
            createFile(tempFileName, fileContant.toString());
        }
        final File tempOrigFile = new File(tempFileName);
        LpgSafetyInspectionExportJob.LOG.info("Inside generateReportCSV tempOrigFile : {} ", Objects.nonNull(tempOrigFile) ? tempOrigFile.getAbsolutePath() : "");
        final String csvPath = this.configurationService.getConfiguration().getString("bpcl.safety.inspection.hybris.input.filepath") + "BPCL_SafetyInspection_" + simpleDateFormat.format(new Date()) + fileExtension;
        LpgSafetyInspectionExportJob.LOG.info("Encrypted CSV File path is {}", csvPath);
        final File encryptedFile = new File(csvPath);
        if (Config.getBoolean("lpg.encryption.enable", false)) {
            LpgSafetyInspectionExportJob.LOG.info("Encrypting the file");
            if (Objects.nonNull(tempOrigFile)) {
                LpgSafetyInspectionExportJob.LOG.info("tempOrigFile not null");
                this.defaultLpgPgpKeys.fileEncryption(encryptedFile, tempOrigFile.getAbsolutePath(), this.configurationService.getConfiguration().getString("lpg.encryption.publickey"));
            }
            safetyInspectionFile = encryptedFile;
        }
        final String targetPath = this.configurationService.getConfiguration().getString("lpg.sftp.safety.inspection.outbound.path");
        this.lpgDeliveryConfirmationService.uploadFileOnSFTP(String.valueOf(encryptedFile), targetPath);
        LpgSafetyInspectionExportJob.LOG.info("CSV File sent to SFTP Successfully and file name is {}", fileName);
        deleteFile(String.valueOf(tempOrigFile));
    }
    
    private void writeCsvHeader(final StringBuilder writer) throws IOException {
        writer.append("InspectionGroupId~DistCode~ConsumerNo~ServiceType~Requestcreatedon~SurakshaTubeExpiry~Latitude~Longitude");
    }
    
    private void writeCsvLine(final StringBuilder stringBuilder, final LpgSafetyInspectionModel safetyInspection) throws IOException, ParseException {
        LpgSafetyInspectionExportJob.LOG.info("Inside writeCsvLine ");
        if (safetyInspection.getId() != null && safetyInspection.getDistributorCode() != null && safetyInspection.getCustomerNumber() != null) {
            stringBuilder.append(safetyInspection.getId());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getDistributorCode());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getCustomerNumber());
            stringBuilder.append("~");
            stringBuilder.append("3");
            stringBuilder.append("~");
            final String creationTime = BPCLDateUtils.dateToString(safetyInspection.getSafetyInspectionDate(), "yyyy-MM-dd 00:00:00");
            stringBuilder.append(creationTime);
            stringBuilder.append("~");
            final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd 00:00:00");
            final Date surakshaDate = dateFormat.parse(safetyInspection.getSurakshaDate());
            final String surakshaExpiryDate = BPCLDateUtils.dateToString(surakshaDate, "yyyy-MM-dd 00:00:00");
            stringBuilder.append(surakshaExpiryDate);
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getLatitude());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getLongitude());
            stringBuilder.append(System.getProperty("line.separator"));
        }
    }
    
    private void writeCsvLineSuraksha(final StringBuilder stringBuilder, final LpgSafetyInspectionModel safetyInspection) throws ParseException {
        LpgSafetyInspectionExportJob.LOG.info("Inside writeCsvLine for Suraksha change");
        if (safetyInspection.getId() != null && safetyInspection.getDistributorCode() != null && safetyInspection.getCustomerNumber() != null) {
            stringBuilder.append(safetyInspection.getId());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getDistributorCode());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getCustomerNumber());
            stringBuilder.append("~");
            stringBuilder.append("2");
            stringBuilder.append("~");
            final String creationTime = BPCLDateUtils.dateToString(safetyInspection.getSafetyInspectionDate(), "yyyy-MM-dd 00:00:00");
            stringBuilder.append(creationTime);
            stringBuilder.append("~");
            final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd 00:00:00");
            final Date surakshaDate = dateFormat.parse(safetyInspection.getSurakshaDate());
            final String surakshaExpiryDate = BPCLDateUtils.dateToString(surakshaDate, "yyyy-MM-dd 00:00:00");
            stringBuilder.append(surakshaExpiryDate);
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getLatitude());
            stringBuilder.append("~");
            stringBuilder.append(safetyInspection.getLongitude());
            stringBuilder.append(System.getProperty("line.separator"));
        }
    }
    
    public static void createFile(final String fileName, final String fileContent) throws IOException {
        FileWriter writer = null;
        try {
            writer = new FileWriter(fileName);
            writer.write(fileContent);
        }
        finally {
            IOUtils.closeStream((Closeable)writer);
        }
    }
    
    public static void deleteFile(final String fileDir) throws IOException {
        final Path path = Paths.get(fileDir, new String[0]);
        Files.newOutputStream(path, StandardOpenOption.DELETE_ON_CLOSE);
    }
    
    public boolean isAbortable() {
        return true;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setLpgDeliveryConfirmationService(final LpgDeliveryConfirmationService lpgDeliveryConfirmationService) {
        this.lpgDeliveryConfirmationService = lpgDeliveryConfirmationService;
    }
    
    public void setBpclSafetyInspectionDao(final BpclSafetyInspectionDao bpclSafetyInspectionDao) {
        this.bpclSafetyInspectionDao = bpclSafetyInspectionDao;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgSafetyInspectionExportJob.class);
        TEMP_DIR = Config.getParameter("bpcl.safety.inspection.hybris.input.filepath");
    }
}
